package com.kob.botrunningsystem.service.impl.utils;

import com.kob.botrunningsystem.utils.BotInterface;
import org.joor.Reflect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.UUID;
import java.util.function.Supplier;

@Component
public class Consumer extends Thread {
    private Bot bot;
    // 定义返回信息 URL
    private final static String receiveBotMoveUrl = "http://127.0.0.1:3001/pk/receive/bot/move/";

    //注入 RestTemplate 用于给 backend 服务发送信息
    private static RestTemplate restTemplate;

    //注入 RestTemplate 用于给 backend 服务发送信息
    @Autowired
    public void setRestTemplate(RestTemplate restTemplate) {
        Consumer.restTemplate = restTemplate;
    }

    // 传入 等待时间 和 bot
    public void startTimeout(long timeout, Bot bot) {
        this.bot = bot;
        this.start();   // 执行线程

        try {
            // 最多阻塞 timeout 时间后继续执行后面的操作(上面的线程最多等待执行 timeout 秒)
            // 相比 sleep 的好处是, join 最多执行 timeout 秒,如果这个早跑完任务就早点结束阻塞; sleep 则必须阻塞够 timeout 才会结束阻塞
            this.join(timeout);
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            this.interrupt();   // 中断当前线程:一旦等待时间超过 timeout 秒,就中断线程
        }
    }

/*
    // 在 Code 中的 Bot 类名后面," implements com.kob.botrunningsystem.utils.BotInterface" 之前加上 uid
    private String addUid(String code, String uid) {
        int k = code.indexOf(" implements com.kob.botrunningsystem.utils.BotInterface");
        return code.substring(0, k) + uid + code.substring(k);
    }
*/

    // 在 Code 中的 Bot 类名后面," implements com.kob.botrunningsystem.utils.BotInterface" 之前加上 uid
    private String addUid(String code, String uid) {
        int k = code.indexOf(" implements java.util.function.Supplier<Integer>");
        return code.substring(0, k) + uid + code.substring(k);
    }

    @Override
    public void run() {
        // 使用 UUID 生成随机字符串
        UUID uuid = UUID.randomUUID();
        String uid = uuid.toString().substring(0, 8);    // 返回前 8 位 uuid 作为 uid

        // Reflect 来自 JOOR　依赖: 可以动态的编译 java 代码
        // 重名类只会编译一遍,为了每次新线程都重新编译,
        // 在类名后加一个随机字符串 UUID(每次返回一个不一样的ID)
 /*
        BotInterface botInterface = Reflect.compile(
                "com.kob.botrunningsystem.utils.Bot" + uid,
                addUid(bot.getBotCode(), uid)
        ).create().get();   // compile(编译) 完这个类之后 create(创建) 一个类,并 get(获取) 到这个类
*/
        // 替换接口类型:修改为文件传输形式
        Supplier<Integer> botInterface = Reflect.compile(
                "com.kob.botrunningsystem.utils.Bot" + uid,
                addUid(bot.getBotCode(), uid)
        ).create().get();   // compile(编译) 完这个类之后 create(创建) 一个类,并 get(获取) 到这个类


        // 将微服务之间的数据包通过文件传输,便于在 docker 容器之间进行数据交互
        File file = new File("input.txt");   // 创建输出文件
        try(PrintWriter fileOut = new PrintWriter(file)){
            fileOut.println(bot.getInput());    // 将 bot.getInput() 输出出去
            fileOut.flush();    // 清空缓冲区
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }

/*
        // 存储获取到的方向
        Integer direction = botInterface.nextMove(bot.getInput());
        System.out.println("move-direction: " + bot.getUserId() + " " + direction);  // 根据动态编译结果返回输入
*/

        // 同步修改为文件传输
        Integer direction = botInterface.get();
        System.out.println("move-direction: " + bot.getUserId() + " " + direction);  // 根据动态编译结果返回输入

        // 打包信息
        MultiValueMap<String, String> data = new LinkedMultiValueMap<>();
        data.add("user_id", bot.getUserId().toString());
        data.add("direction", direction.toString());
        // 返回结果信息给 backend 服务端
        restTemplate.postForObject(receiveBotMoveUrl, data, String.class);
    }
}
